"""
#
#    Written by Phil Dodderidge <pdodde@poyntz.com>
#    Copyright (C) 2001 Poyntz, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
"""

/* This program is designed to pass a query to a JDBC Driver
   and return the results. */

/***
 *** IMPORT YOUR DRIVER HERE
 ***/
import com.ibm.as400.access.AS400JDBCDriver;

import java.sql.*;
import java.net.*;
import java.io.*;

public class JDBCServer extends Thread {
    
    /***
     *** CHANGE THE URL FOR YOUR DRIVER
     ***/
    static String db_url = "jdbc:as400://";  // This is the URL that gets passed to the Driver Manager
    static int sockbufsize = 1024;          
    private Socket sock;
    private Connection dbcon;
    private String id;

    public JDBCServer(Socket sock, String id) {
	this.sock = sock;
	this.id = id;
    }

    public static void main(String args[]) throws java.sql.SQLException {

	/***
	 ***  REGISTER YOUR DRIVER HERE
	 ***/
	java.sql.DriverManager.registerDriver( new com.ibm.as400.access.AS400JDBCDriver() );

	try {
	    // Change 127.0.0.1 if you need to make connections from a remote host
	    // Beware of security holes if you change this.
	    ServerSocket sock = new ServerSocket(9494, 5, InetAddress.getByName("127.0.0.1"));
	    int tnum = 1;

	    // Loop to accept incoming connections
	    while(true) {
		String newsock = new Integer(tnum).toString();
		Socket csock = sock.accept();
		Thread t = new JDBCServer(csock, newsock);
		t.start();
		tnum++;
	    }

	} catch (IOException e) {
	    System.out.println("Failed to get Socket Connection");
	    System.err.println(e);
	}
	
	System.exit(0);

    }

    public void run() {
	// This is the thread that does all the work
	InputStream in;
	OutputStreamWriter out;
	String host, user, pw;

	// Get Authentication information to pass to the Driver Manager
	try {
	    in = new BufferedInputStream(this.sock.getInputStream());
	    out = new OutputStreamWriter(this.sock.getOutputStream());
	    out.write("AUTH\n");
	    out.flush();
	    StringBuffer req = new StringBuffer(sockbufsize);
	    for (int ch = in.read(); ch > 0 && ch != '\r' && ch != '\n'; ch = in.read()) {
		req.append((char) ch);
	    }
	    int p = 0;
	    StringBuffer tmp = new StringBuffer(sockbufsize);
	    String[] flds = new String[7]; 
	    for (int i=0; i < req.length(); i++) {
		if (req.charAt(i) == ' ') { 
		    flds[p] = tmp.toString();
		    p++;
		    tmp = new StringBuffer(sockbufsize);
		} else {
		    tmp.append(req.charAt(i));
		}
	    }
	    flds[p] = tmp.toString();
	    if (flds.length >= 6 && flds[0].equals("HOST") && 
		flds[2].equals("USER") && flds[4].equals("PASS")) {
		host = flds[1];
		user = flds[3];
		pw = flds[5];
	    } else {
		return;
	    }
	    out.write("OK\n");
	    out.flush();
	} catch (IOException e) {
	    System.err.println("Can't get input stream\n"+e);
	    return;
	}

	// Create a database connection
	try {
	    this.dbcon = DriverManager.getConnection(db_url+host, user, pw);
	} catch (SQLException e) {
	    System.err.println("Error getting connection "+e);
	    return;
	}

	// Loop to listen for queries
	while(true) {
	    
	    StringBuffer req = new StringBuffer(sockbufsize);
	    Statement s;
	    ResultSet rs;

	    // Create a new statement
	    try {
		s = this.dbcon.createStatement();
	    } catch (SQLException e) {
		System.err.println("Unable to create SQL Statement\n"+e);
		break;
	    }
	    
	    // Read input from the client
	    while(true) {
		try {
		    int ch = in.read();
		    if (ch == '\r' || ch == '\n' || ch == -1) break;
		    req.append( (char) ch );
		} catch (IOException e) {
		    System.err.println("Error reading input stream\n"+ e);
		    break;
		}
	    }
	    
	    
	    StringBuffer retval = new StringBuffer(sockbufsize);
	    try {
		// input must be like "QUERY select * from data"
		if (req.toString().startsWith("QUERY ")) {
		    // Chop off "QUERY "
		    String query = req.toString().substring(6);

		    // Run the query
		    rs = s.executeQuery(query);
		    ResultSetMetaData rsmd = rs.getMetaData();

		    // Return the result in a parsable format
		    int cols = rsmd.getColumnCount();
		    for(int col=1; col<cols; col++) {
			retval.append("<column>");
			retval.append(rsmd.getColumnName(col)+"\t");
			retval.append(rsmd.getColumnTypeName(col)+"\t");
			retval.append(rsmd.getColumnDisplaySize(col)+"\t");
			if(rsmd.isNullable(col) == ResultSetMetaData.columnNullable) {
			    retval.append('1');
			} else {
			    retval.append('0');
			}
			retval.append("</column>");
		    }
		    retval.append('\n');
		    while(rs.next()) {
			retval.append("<row>");
			for(int col=1; col<cols; col++) {
			    retval.append(rs.getString(rsmd.getColumnName(col)));
			    retval.append('\t');
			}
			retval.append("</row>\n");
		    }
		} else {
		    retval.append("ERROR Not a query");
		}
	    } catch (SQLException e) {
		// Catch SQL errors without killing the thread
		retval.append("ERROR "+e.toString());
	    }
	    
	    // Write the output to the socket
	    try {
		out.write(retval.toString()+"\n<EOF>");
		out.flush();
	    } catch (IOException e) {
		try {
		    s.close();
		    this.dbcon.close();
		} catch (SQLException err) {
		    System.err.println("Error Closing Socket\n"+err);
		}
		break;
	    }
	    
	}
    }
    
}

