"""
#
#    Written by Phil Dodderidge <pdodde@poyntz.com>
#    Copyright (C) 2001 Poyntz, Inc.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#    This product includes software developed by Digital Creations
#    for use in the Z Object Publishing Environment
#    (http://www.zope.org/).
#
"""

from Shared.DC.ZRDB.TM import TM
import string
import socket
import exceptions
import re

class DB(TM):
    def __init__(self, connection_string=''):
        """ Connect to the Database """
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        (self.host, self.user, self.pw) = string.split(connection_string, ' ')
        self.recon()

    def _begin(self):
        """ Begin a Transaction """
        pass
    
    def _finish(self):
        """ Standard for a COMMIT """
        pass
    
    def _abort(self):
        """ Standard for a ROLLBACK """
        pass
    
    def recon(self):
        """ Reconnect to the server """
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.connect("127.0.0.1", 9494)

        got = self.sock.recv(1024)
        ret = got
        while(not re.search(r'\n', got) and got):
            got = self.sock.recv(1024)
            ret = ret+got
            
        if ret[0:4] == "AUTH":
            self.sock.send("HOST %s USER %s PASS %s\n" % (self.host, self.user, self.pw))
        else:
            raise exceptions.IOError, "Unable to get Connection"

        got = self.sock.recv(1024)
        ret = got
        while(not re.search(r'\n', got) and got):
            got = self.sock.recv(1024)
            ret = ret+got

        if ret[0:2] != "OK":
            raise exceptions.IOError, "Authentication Failed"
        

    def query(self, src=None, max_rows=99999999):
        """ Search Interface """

        # register with the transaction machinery
        self._register()

        # take the newlines out of the query
        src = re.sub(r'\n',  ' ', src)

        try:
            # send the query
            self.sock.send("QUERY %s\n" % src)
            # get the result
            got = self.sock.recv(1024)
        except:
            # try to reconnect and send query again
            self.recon()
            self.sock.send("QUERY %s\n" % src)
            got = self.sock.recv(1024)

        # read the result
        ret = got
        while(not re.search(r'<EOF>$', got) and got):
            got = self.sock.recv(1024)
            ret = ret+got

        # check for SQL Error
        if re.match(r'ERROR', ret):
            raise exceptions.IOError, ret[6:]

        # parse the results
        ret_items = []
        for col in re.findall(r'<column>(.*?)</column>',ret):
            colvals = string.split(col, '\t')
            ret_items.append({'name': colvals[0],
                          'type': 's',        # can be s (String), d (Date), i (Integer), or n (Number)
                          'width': colvals[2],
                          'null': colvals[3]},)
        ret_result = []
        for row in re.findall("<row>(.*?)</row>", ret):
            ret_result.append(string.split(row, '\t'))

        return (ret_items, ret_result)
